<?php

namespace Drupal\commerce_colissimo_shipping\Plugin\Commerce\ShippingMethod;

use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping\Plugin\Commerce\ShippingMethod\ShippingMethodBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\PackageTypeManagerInterface;
use Drupal\commerce_shipping\ShippingRate;
use Drupal\commerce_shipping\ShippingService;
use Drupal\physical\MeasurementType;
use Drupal\physical\Calculator;

/**
 * Provides the Colissimo shipping method.
 *
 * @CommerceShippingMethod(
 *   id = "colissimo_shipping",
 *   label = @Translation("Colissimo shipping"),
 * )
 */
class Colissimo extends ShippingMethodBase {

  /**
   * Constructs a new Colissimo object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_shipping\PackageTypeManagerInterface $package_type_manager
   *   The package type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PackageTypeManagerInterface $package_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $package_type_manager);

    $this->plgin_d = $plugin_definition;
    $this->services['default'] = new ShippingService('default', $this->configuration['colis_set']['rate_label']);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {

    $build['colis_set'] = [
      'rate_label' => NULL,
      'rate_amount' => NULL,
      'min_weight' => NULL,
      'max_weight' => NULL
    ];
    $build['services'] = ['default'];

    return $build + parent::defaultConfiguration();

  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $amount = $this->configuration['colis_set']['rate_amount'];
    // A bug in the plugin_select form element causes $amount to be incomplete.
    if (isset($amount) && !isset($amount['number'], $amount['currency_code'])) {
      $amount = NULL;
    }

    $form['colis_set'] = array(
      '#type' => 'details',
      '#title' => $this->t('Colissimo settings'),
      '#description' => $this->t('Select your colissimo setting.'),
      '#open' => !$this->isConfigured(),
    );

    $form['colis_set']['rate_label'] = [
      '#type' => 'textfield',
      '#title' => t('Rate label'),
      '#description' => t('Shown to customers during checkout.'),
      '#default_value' => $this->configuration['colis_set']['rate_label'],
      '#required' => TRUE,
    ];
    $form['colis_set']['min_weight'] = [
      '#type' => 'physical_measurement',
      '#title' => $this->t('Minimun weight'),
      '#measurement_type' => 'weight',
      '#default_value' => $this->configuration['colis_set']['min_weight'],
      '#min' => 0,
      '#max' => 100,
    ];
    $form['colis_set']['max_weight'] = [
      '#type' => 'physical_measurement',
      '#title' => $this->t('Maximun weight'),
      '#measurement_type' => 'weight',
      '#default_value' => $this->configuration['colis_set']['max_weight'],
      '#min' => 0,
      '#max' => 100,
    ];
    $form['colis_set']['rate_amount'] = [
      '#type' => 'commerce_price',
      '#title' => t('Rate amount'),
      '#default_value' => $amount,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['colis_set']['rate_label'] = $values['colis_set']['rate_label'];
      $this->configuration['colis_set']['rate_amount'] = $values['colis_set']['rate_amount'];
      $this->configuration['colis_set']['min_weight'] = $values['colis_set']['min_weight'];
      $this->configuration['colis_set']['max_weight'] = $values['colis_set']['max_weight'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function calculateRates(ShipmentInterface $shipment) {
    // Rate IDs aren't used in a flat rate scenario because there's always a
    // single rate per plugin, and there's no support for purchasing rates.
    $rate_id = 0;
    $min_weight = $this->configuration['colis_set']['min_weight'];
    $max_weight = $this->configuration['colis_set']['max_weight'];

    $quantity = 0;
    foreach ($shipment->getItems() as $shipment_item) {
      $quantity += $shipment_item->getQuantity();
    }

    $weight_total = 0;
    foreach ($shipment->getItems() as $shipment_item) {
      $unit = $shipment_item->getWeight()->getUnit();
      if ($unit != 'kg') {
        $weight_kg = $shipment_item->getWeight()->convert('kg')->getNumber();
        $weight_total += $weight_kg;
      }
      else {
        $weight_kg = $shipment_item->getWeight()->getNumber();
        $weight_total += $weight_kg;
      }
    }

    $amount = $this->configuration['colis_set']['rate_amount'];
    $amount = new Price($amount['number'], $amount['currency_code']);
    $amount = $amount->multiply((string) $weight_total);

    $rates = [];
    $min_weight_kg = $this->_convert($min_weight['unit'], 'kg', 'weight', $min_weight['number'], 4);
    $max_weight_kg = $this->_convert($max_weight['unit'], 'kg', 'weight', $max_weight['number'], 4);

    if ( $weight_total >= $min_weight_kg['number'] && $weight_total < ($max_weight_kg['number']) ) {
      drupal_set_message(
        t(
          'Tengo un total de @weight_total kg, pertenezco a «@label» rate y soy mayor o igual que @min_weight_kg kg pero menor que @max_weight_kg kg',
          [
            '@weight_total' => $weight_total,
            '@min_weight_kg' => !empty($min_weight_kg) ? $min_weight_kg['number'] : '0',
            '@max_weight_kg' => $max_weight_kg['number'],
            '@label' => $this->configuration['colis_set']['rate_label']
          ]
        ),
        'status'
      );
      $rates[] = new ShippingRate($rate_id, $this->services['default'], $amount);
    }
    else {
      drupal_set_message(
        t(
          'Tengo un total de @weight_total kg y pertenezco a «@label» rate, pero no pertenezco al rango entre @min_weight_kg kg y @max_weight_kg kg',
          [
            '@weight_total' => $weight_total,
            '@min_weight_kg' => !empty($min_weight_kg) ? $min_weight_kg['number'] : '0',
            '@max_weight_kg' => $max_weight_kg['number'],
            '@label' => $this->configuration['colis_set']['rate_label']
          ]
        ),
        'warning'
      );

    }

    return $rates;
  }

  /**
   * Converts the current measurement to a new unit.
   *
   * @param string $new_unit
   *   The new unit.
   *
   * @return static
   *   The resulting length.
   */
  public function _convert($current_unit, $new_unit, $unit_type, $current_number, $scale = 2) {
    /* @var \Drupal\physical\UnitInterface $units_class */
    $units_class = MeasurementType::getUnitClass($unit_type);

    // Convert the number to the base unit, then from there to the new unit.
    $base_number = Calculator::multiply($current_number, $units_class::getBaseFactor($current_unit));
    $new_number = Calculator::divide($base_number, $units_class::getBaseFactor($new_unit), $scale);

    // return new static($new_number, $new_unit);
    return [
      'number' => $new_number,
      'unit' => $new_unit
    ];
  }

  /**
   * Determine if we have the minimum information to connect to NZPost.
   *
   * @return bool
   *   TRUE if there is enough information to connect, FALSE otherwise.
   */
  protected function isConfigured() {
    $api_information = $this->configuration['colis_set'];

    if (!empty($api_information['rate_amount'])) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

}
